@@ -27,6 +27,7 @@ import calendar
 from itertools import groupby
 from gettext import ngettext
 import locale
+import math
 
 import gtk, gobject
 import pango
@@ -124,35 +125,25 @@ class Stats(object):
 
         #ah, just want summary look just like all the other text on the page
         class CairoText(graphics.Scene):
-            def __init__(self, fontsize = 10):
+            def __init__(self):
                 graphics.Scene.__init__(self)
                 self.text = ""
-                self.fontsize = fontsize
+                self.label = graphics.Label(self.text, 10)
+                self.label.wrap = pango.WRAP_WORD
+                self.add_child(self.label)
+                self.connect("on-enter-frame", self.on_enter_frame)
 
             def set_text(self, text):
-                self.text = text
+                self.label.text = text
                 self.redraw()
 
-            def on_expose(self):
+            def on_enter_frame(self, scene, context):
                 # now for the text - we want reduced contrast for relaxed visuals
                 fg_color = self.get_style().fg[gtk.STATE_NORMAL].to_string()
-                if self.colors.is_light(fg_color):
-                    label_color = self.colors.darker(fg_color,  80)
-                else:
-                    label_color = self.colors.darker(fg_color,  -80)
+                self.label.color = self.colors.contrast(fg_color,  80)
 
-                default_font = pango.FontDescription(gtk.Style().font_desc.to_string())
-                default_font.set_size(self.fontsize * pango.SCALE)
-                self.layout.set_font_description(default_font)
+                self.label.width = self.width
 
-                #self.context.set_source_rgb(0,0,0)
-                self.layout.set_markup(self.text)
-
-                self.layout.set_width((self.width) * pango.SCALE)
-                self.context.move_to(0,0)
-                self.set_color(label_color)
-
-                self.context.show_layout(self.layout)
 
         self.explore_summary = CairoText()
         self.get_widget("explore_summary").add(self.explore_summary)
@@ -173,7 +164,7 @@ class Stats(object):
                 label.set_text(_("""There is no data to generate statistics yet.
 A week of usage would be nice!"""))
             else:
-                label.set_text(_("Collecting data — check back after a week has passed!"))
+                label.set_text(_(u"Collecting data — check back after a week has passed!"))
 
             label.show()
             return
@@ -237,8 +228,21 @@ A week of usage would be nice!"""))
 
 
         for day in by_weekday:
-            by_weekday[day] = (sum([fact[0] for fact in by_weekday[day]]) / len(by_weekday[day]),
-                               sum([fact[1] for fact in by_weekday[day]]) / len(by_weekday[day]))
+            n = len(by_weekday[day])
+            # calculate mean and variance for starts and ends
+            means = (sum([fact[0] for fact in by_weekday[day]]) / n,
+                     sum([fact[1] for fact in by_weekday[day]]) / n)
+            variances = (sum([(fact[0] - means[0]) ** 2 for fact in by_weekday[day]]) / n,
+                         sum([(fact[1] - means[1]) ** 2 for fact in by_weekday[day]]) / n)
+
+            # In the normal distribution, the range from
+            # (mean - standard deviation) to infinit, or from
+            # -infinit to (mean + standard deviation),  has an accumulated
+            # probability of 84.1%. Meaning we are using the place where if we
+            # picked a random start(or end), 84.1% of the times it will be
+            # inside the range.
+            by_weekday[day] = (int(means[0] - math.sqrt(variances[0])),
+                               int(means[1] + math.sqrt(variances[1])))
 
         min_weekday = min([by_weekday[day][0] for day in by_weekday])
         max_weekday = max([by_weekday[day][1] for day in by_weekday])
@@ -278,8 +282,15 @@ A week of usage would be nice!"""))
                     by_category[category].append((min(start_times), max(end_times)))
 
         for cat in by_category:
-            by_category[cat] = (sum([fact[0] for fact in by_category[cat]]) / len(by_category[cat]),
-                                sum([fact[1] for fact in by_category[cat]]) / len(by_category[cat]))
+            # For explanation see the comments in the starts and ends by day
+            n = len(by_category[cat])
+            means = (sum([fact[0] for fact in by_category[cat]]) / n,
+                     sum([fact[1] for fact in by_category[cat]]) / n)
+            variances = (sum([(fact[0] - means[0]) ** 2 for fact in by_category[cat]]) / n,
+                         sum([(fact[1] - means[1]) ** 2 for fact in by_category[cat]]) / n)
+
+            by_category[cat] = (int(means[0] - math.sqrt(variances[0])),
+                                int(means[1] + math.sqrt(variances[1])))
 
         min_category = min([by_category[day][0] for day in by_category])
         max_category = max([by_category[day][1] for day in by_category])
